package docex;

import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.Vector;


public class Article implements Comparable, TFIDF {
	public String orig_input_string = new String();
	public String text = new String();
	public Vector<Paragraph> paragraphs = new Vector<Paragraph>();
	public String id = new String();
	public String headline = new String();
	private boolean enabled = true;
	
	/* TFIDF - Interface */
	//@TODO make private
	public HashMap<String,Integer> word_frequencies = new HashMap<String,Integer>();
	public HashMap<String,Double> tfidf = new HashMap<String,Double>();
	public Double cosine_similarity = (double) 0;
	public Double dotproduct_similarity = (double)0;
	public boolean sortByDate = false;
	public Date publicationDate;
	
	
	/**
	 * Taking a TEXT-segment as input to initialize the articles text.
	 * The only other XML-Tags expected are <P> separating paragraphs.
	 * Removing paragraphs does not change (this.text), it  will always 
	 * contain the raw article text
	 * @param content
	 */
	public Article(String content) {
		orig_input_string = content;
		int start=0,end=0;
		int pnum = 0;

		if (content.contains("id=")){
			start = content.indexOf("id=")+4;
			end = content.indexOf("\"",start);
			id = content.substring(start,end);
		} else {
			id = "undef";
		}

		try {
			publicationDate = Tools.getDateFromID(id);
		} catch (ParseException e) {
			e.printStackTrace();  // unhandled (all articles should have their ID set)
			System.exit(0);
		}
		
		if (content.contains("headline=")){
			start = content.indexOf("headline=")+10;
			end = content.indexOf("\"",start);
			headline = content.substring(start,end);
		} else {
			headline = "";
		}
		
		start=0; end=0;
		do {
			if ((end-start)>0) {
				pnum++;
				String p = content.substring(start+3, end);
				text += p;
				text += " ";
				paragraphs.add(new Paragraph(this,pnum,p));
			}
			start = content.indexOf("<P>", end);
			end = content.indexOf("</P>", start);
		} while ((start != -1) & (end != -1));
/*
 *  Have to do this differently due to hyphens at end of line, will produce "Carl- Marx" 
 *		text.replace('\n',' ');  // no linefeeds in text
 */
		
	}
	
	
	public void setSortByDate() {
		sortByDate = true;
	}

	public void setSortBySimilarity() {
		sortByDate = false;
	}
	
	public boolean equals(Article a) {
		return a.text.equals(text);
	}
	
	@Override
	/**
	 * This is used for sorting a collection of articles by 
	 * their similarity rating. (this.cosine_similarity)
	 */
	public int compareTo(Object o) {
		if (o instanceof Article) {
			Article a = (Article)o;

			if (sortByDate)
//				return publicationDate.compareTo(a.publicationDate);
				return id.compareTo(a.id);
			else
				return Double.compare(cosine_similarity, a.cosine_similarity);  // sort by cosine_similarity
		} else {
			return 0;
		}
	}
	
	/**
	 * Concats all paragraphs into a string with double linefeeds
	 * after each paragraph. (This is not the same as this.text)
	 * @return
	 */
	public String getReadableParagraphs() {
		String out = new String();
		for (Paragraph p : paragraphs) {
			out += p.getText();
			out += "\n\n";
		}
		return out;
	}

	public boolean isEmpty() {
		return paragraphs.isEmpty();
	}

	
	public String getXMLString() {
		String xml = new String();
		String sim_str = String.format("%.3f", cosine_similarity);
		
		xml = "\n<TEXT cosine_similarity="+sim_str+" id=\""+id+"\" headline=\""+headline+"\"";
		xml += ">";
		
		for (Paragraph p : paragraphs) {
			xml += "<P>\n";
			xml += p.getText();
			xml += "\n</P>";
		}
		
		xml += "</TEXT>\n";
		
		return xml;
	}
	
	
	/**
	 * Returns the percentage of text in this article that is
	 * enabled (based on paragraph size, not just counting paragraphs).
	 * @return
	 */
	public double enabledPercentage() {
		double pct = 0.0;
		int enabled = 0;
		int overall = 0;
		
		for (Paragraph p : paragraphs) {
			if (p.isEnabled())
				enabled += p.word_frequencies.size();
			overall += p.word_frequencies.size();
		}
		
		pct = enabled;
		pct = pct / ((double)overall);
		
		return pct;
	}

	
	
	/* TFIDF - Interace */

	@Override
	public HashMap<String, Integer> getWord_frequencies() {
		return word_frequencies;
	}

	@Override
	public void setWord_frequencies(HashMap<String, Integer> wf) {
		word_frequencies = wf;
	}

	@Override
	public HashMap<String, Double> getTfidf() {
		return tfidf;
	}

	@Override
	public void setTfidf(HashMap<String, Double> newtfidf) {
		tfidf = newtfidf;
	}

	@Override
	public Double getCosine_similarity() {
		return cosine_similarity;
	}

	@Override
	public void setCosine_similarity(Double cosim) {
		cosine_similarity = cosim;
	}

	@Override
	public Double getDotproduct_similarity() {
		return dotproduct_similarity;
	}
	
	@Override
	public void setDotproduct_similarity(Double dotsim) {
		dotproduct_similarity = dotsim;
	}

	public boolean isEnabled() {
		return enabled;
	}
	
	public void enable() { enabled = true; }
	public void disable() { enabled = false; }
	
}
