package docex;


import java.util.ArrayList;
import java.util.TreeMap;

import gurobi.*;


public class Optimizer {

	private GRBEnv    env;
	private GRBModel  model;
	private Integer next_par_id;
	private TreeMap<Integer,ArrayList<Paragraph> > parmap;
	
	
	public Optimizer () {
		parmap = new TreeMap<Integer,ArrayList<Paragraph> >();
		next_par_id = 1;
		try {
			env   = new GRBEnv("qp.log");
			env.set(GRB.DoubleParam.TimeLimit, 60.0); // set the optimization limit in seconds
			model = new GRBModel(env);
		} catch (GRBException e) {
			System.out.println("Error code: " + e.getErrorCode() + ". " + e.getMessage());
			System.exit(-1);
		}
	}
	
	

	public void addVar(Paragraph par)  {
		if (par.distinct_par_id == 0) {
			par.distinct_par_id = next_par_id;
			next_par_id++;
			ArrayList<Paragraph> plist = new ArrayList<Paragraph>();
			plist.add(par);
			parmap.put(par.distinct_par_id,plist);
		} else {
			parmap.get(par.distinct_par_id).add(par);
		}
		try {
			par.opt_var = model.addVar(0, 1, 0, GRB.INTEGER, par.getParId());
		} catch (GRBException e) {
			e.printStackTrace();
			System.exit(-1);
		}
	}
	
	


	public void optimize_article_completenes(ArrayList<Article> articles) {
		try {

			System.out.println("optimizing ...");
			
			model.update();

			// generate all the constraint terms accross the similar paragraph-lists
			for (Integer k : parmap.keySet()) {
				GRBLinExpr expr = new GRBLinExpr();
				ArrayList<Paragraph> plist = parmap.get(k);
				for (Paragraph par : plist) {
					// Add constraint: a1 + a2 <= 1
					expr.addTerm(1, par.opt_var);
				}
				// add constraint with its id on the map
				model.addConstr(expr, GRB.LESS_EQUAL, 1, "constr"+k);
			}
			
			GRBQuadExpr obj = new GRBQuadExpr();
			obj.clear();

			System.out.println("going over "+articles.size()+" articles");
			
			// generate and add the quadratic objective function
			for (Article a : articles) {
				int i,j,size = a.paragraphs.size();
				for (i=0; i<size; i++) {
					for (j=i; j<size; j++) {
						if (i==j) {
							// add the (a^2 + b^2 + c^2 ...) component
							Paragraph p = a.paragraphs.get(i);
							obj.addTerm(1*p.word_frequencies.size()*p.word_frequencies.size(), p.opt_var, p.opt_var); 
						} else {
							// add the (2ab + 2ac + 2bc ...) component
							Paragraph pi = a.paragraphs.get(i);
							Paragraph pj = a.paragraphs.get(j);
							obj.addTerm(2*pi.word_frequencies.size()*pj.word_frequencies.size(), a.paragraphs.get(i).opt_var, a.paragraphs.get(j).opt_var); 
						}
					}
				}
			}
						
			// add the objective function and its target MAXIMIZATION
			model.setObjective(obj,GRB.MAXIMIZE);
			

			model.optimize();
			
		} catch (GRBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Optimizer o = new Optimizer();
		
		o.sample_optimize();
	}
	
	
	public void sample_optimize () {

		try {
			GRBEnv    env   = new GRBEnv("qp.log");
			GRBModel  model = new GRBModel(env);
			
			// Create variables

			GRBVar a1 = model.addVar(0, 1, 0, GRB.INTEGER, "a1");
			GRBVar b1 = model.addVar(0, 1, 0, GRB.INTEGER, "b1");
			GRBVar c1 = model.addVar(0, 1, 0, GRB.INTEGER, "c1");
			
			GRBVar a2 = model.addVar(0, 1, 0, GRB.INTEGER, "a2");
			GRBVar b2 = model.addVar(0, 1, 0, GRB.INTEGER, "b2");
			GRBVar c2 = model.addVar(0, 1, 0, GRB.INTEGER, "c2");

			// Integrate new variables
			model.update();

			GRBQuadExpr obj = new GRBQuadExpr();
			obj.addTerm(1, a1, a1);
			obj.addTerm(6, a1, b1);
			obj.addTerm(2, a1, c1);
			obj.addTerm(9, b1, b1);
			obj.addTerm(6, b1, c1);
			obj.addTerm(1, c1, c1);
			
			obj.addTerm(1, a2, a2);
			obj.addTerm(6, a2, b2);
			obj.addTerm(9, b2, b2);
			
			model.setObjective(obj,GRB.MAXIMIZE);


			// Add constraint: a1 + a2 <= 1
			GRBLinExpr expr = new GRBLinExpr();
			expr.addTerm(1, a1); expr.addTerm(1, a2);
			model.addConstr(expr, GRB.LESS_EQUAL, 1, "c0");  // constraint 0
			
			// Add constraint: b1 + b2 <= 1
			expr.clear();
			expr.addTerm(1, b1); expr.addTerm(1, b2);
			model.addConstr(expr, GRB.LESS_EQUAL, 1, "c0");  // constraint 0

			// Add constraint: c1 + c2 <= 1
			expr.clear();
			expr.addTerm(1, c1); expr.addTerm(1, c2);
			model.addConstr(expr, GRB.LESS_EQUAL, 1, "c0");  // constraint 0

			

			System.out.println(a1.get(GRB.StringAttr.VarName)
					+ " " +a1.get(GRB.DoubleAttr.X));
			System.out.println(b1.get(GRB.StringAttr.VarName)
					+ " " +b1.get(GRB.DoubleAttr.X));
			System.out.println(c1.get(GRB.StringAttr.VarName)
					+ " " +c1.get(GRB.DoubleAttr.X));

			System.out.println(a2.get(GRB.StringAttr.VarName)
					+ " " +a2.get(GRB.DoubleAttr.X));
			System.out.println(b2.get(GRB.StringAttr.VarName)
					+ " " +b2.get(GRB.DoubleAttr.X));
			System.out.println(c2.get(GRB.StringAttr.VarName)
					+ " " +c2.get(GRB.DoubleAttr.X));

			System.out.println("Obj: " + model.get(GRB.DoubleAttr.ObjVal) + " " +
					obj.getValue());



			
			// Dispose of model and environment

			model.dispose();
			env.dispose();   

		} catch (GRBException e) {
			System.out.println("Error code: " + e.getErrorCode() + ". " +
					e.getMessage());
		}
	}

	
}
