package docex;

import gurobi.GRB;
import gurobi.GRBException;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.StringReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;




public class StoryVector { 

	// a list containing ALL paragraphs seen along the time axis (growing)
	private TreeMap<Date,DailyCollection> storyVector = new TreeMap<Date,DailyCollection>(); 
	private Optimizer optimizer = new Optimizer();

	private Vector<Vector<Vector<Paragraph>>> sample_paragraph_pairs = new Vector<Vector<Vector<Paragraph>>>();
	private boolean GEN_STRAT_SAMPLES = true;
	private int PER_SLOT_COUNT = 30;

	// fixed definition of thresholds
	private Double MIN_PERCENT_TO_ENABLE_ARTICLE = 0.80;
	private Double PARAGRAPH_SIMILARITY_THRESHOLD = 0.5;
	
	HashMap<String,Double> corpus_idf = new HashMap<String,Double>();
	int[] simstats = new int[105];

	public void initialize() {
		ObjectInputStream in;
		System.out.print("initializing corpus_idf ... ");
		for (int i=0; i<105; i++)
			simstats[i]=0;
		for (int i=0; i<22; i++) {
			sample_paragraph_pairs.add(new Vector<Vector<Paragraph>>());
		}
		try {
			in = new ObjectInputStream(new FileInputStream("word_idf.jserial"));
			corpus_idf = (HashMap<String,Double>) in.readObject();
			System.out.println("ArticleSelector.initialize() done "+corpus_idf.size()+" words read");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("done");
	}
	

	
	
	
	/**
	 * Orders articles by time. Strips duplicate paragraphs by removing the paragraph 
	 * that appears later in time. And groups them by day.
	 * @param source List of input-articles to be organized into the StoryVector
	 */
	public void parseStories(LinkedList<Article> source, Date start) {
		BufferedReader br;
		Statistics stats = new Statistics(false);
		
		Date d = new Date();
		SimpleDateFormat ft = new SimpleDateFormat ("yyyy-MM-dd"); 
		try { 
			d = ft.parse("1994-01-01"); 
		} catch (ParseException e) { 
			System.out.println("Unparseable using " + ft); 
		}

		System.out.println("parsing stories");
		
		for (Article a : source)
			a.setSortByDate();
		Collections.sort(source);
		
		/* walk over all input-articles */
		for (Article rtcl : source) {
			if (rtcl.publicationDate.compareTo(start) < 0)
				continue;
			if (rtcl.paragraphs.size() <= 1) // drop one-line-articles
				continue;
			Iterator<Paragraph> p_iter = rtcl.paragraphs.iterator();
			Paragraph par;
			
			System.out.println("article "+rtcl.id);
			
			/* walk over each paragraph per article */
			while (p_iter.hasNext()) {
				par = p_iter.next();
				
				// create per-paragraph stats (and count multiple occurrences)
				br = new BufferedReader(new StringReader(par.getText()));
				stats.readCorpus(br, false);
				for (Integer i : stats.id_to_word.keySet()) {
					String word = stats.id_to_word.get(i);
					par.word_frequencies.put(word, stats.id_to_count.get(i));
				}
				par.tfidf = Tools.TFIDFfromWordFrequency(corpus_idf,par.word_frequencies);
				stats.resetStatistics();
				
				// generate TFIDF-table per paragraph
				par.setTfidf(Tools.TFIDFfromWordFrequency(corpus_idf, par.getWord_frequencies()));
				
				handleDuplicates(par);
			}
			
			// If there is new text in this article then add to the collection
			if (!rtcl.isEmpty()) {
				try {
					d = Tools.getDateFromID(rtcl.id);
					add(d, rtcl);
				} catch (ParseException e) {
					System.out.println("Exception parsing date string!! "+rtcl.id);
					// if parsing does not work or the id was empty the date
					// of the last article is still in d (-> gets added to todays batch)
				}
			}
		}
		
		if (GEN_STRAT_SAMPLES) {
			
			FileWriter fw_eval;
			try {
				fw_eval = new FileWriter("paragraph_level_evaluation.txt",false);
				fw_eval.write("Paragraphs similarity evaluation\n");
				fw_eval.write("(similarity is given as a factor of 100, 0>sim>100, ie. a similarty of 0.23 would result in sim=23)\n");
				for (int i=0; i<20; i++) {
					Vector<Vector<Paragraph>> v = sample_paragraph_pairs.get(i);
					for (Vector<Paragraph> pp : v) {
						fw_eval.write("Paragraphs (sim="+i*5+"):  "+pp.get(0).getParId()+"_#"+pp.get(0).pnum);
						fw_eval.write("  /  "+pp.get(1).getParId()+"_#"+pp.get(1).pnum+" \n");
						fw_eval.write(pp.get(0).getText());
						fw_eval.write("\n-------------\n");
						fw_eval.write(pp.get(1).getText());
						fw_eval.write("\n\n");
					}
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
	}

	
	
	/**
	 * Add a paragraph to the given Date (collection) of this StoryVector
	 * @param d
	 * @param a
	 */
	public void add(Date d, Article a) {
		if (storyVector.containsKey(d)) {
			storyVector.get(d).articles.add(a);
		} else {
			DailyCollection dc = new DailyCollection();
			dc.date = d;
			dc.articles.add(a);
			storyVector.put(d, dc);
		}
	}
	
	
	/**
	 * Check if the given article is considered a near duplicate of
	 * an article in this StoryVector
	 * @param par
	 * @return
	 */
	public void handleDuplicates(Paragraph new_par) {
		Statistics stats = new Statistics(false);
		BufferedReader br;
		Double sim = 0.0;
		boolean have_seen_duplicate = false;

		Double topsim = 0.0;
		Paragraph toppar = null;
		
		ArrayList<Date> date_list = new ArrayList<Date>(storyVector.keySet());
		Collections.sort(date_list);

		
		
		/*
		 * At this point we want to go back from the article that is about to be added
		 * to the StoryVector. Going in reverse order over all the collection only works
		 * because at the point of calling isNearDuplicate() there are only articles up
		 * to this date in the collection. When called again later this function would
		 * return different results.
		 */
		for (Date d : Reversed.reversed(date_list)) {
			for (Article a : Reversed.reversed(storyVector.get(d).articles)) {
				for (Paragraph a_par : a.paragraphs) {
					sim = Tools.similarityFromTFIDFVectors(new_par,a_par);
					simstats[(int)Math.floor(sim*100)]++;
					String str_sim = String.format("similarity is %.3f",sim);
					
					// Generate stratified samples if requested
					if (GEN_STRAT_SAMPLES) {
						if (Math.random() < 0.5) {  // get some %age
							int index = (int)Math.floor(sim*20);
							if (index > 20) index=20; // check sanity
							if (sample_paragraph_pairs.get(index).size() < PER_SLOT_COUNT) {
								Vector<Paragraph> tmp = new Vector<Paragraph>(2);
								tmp.add(a_par);
								tmp.add(new_par);
								sample_paragraph_pairs.get(index).add(tmp);
							}
						}
					}

					// if this paragraph is subsumed by a previous one (a real subset, not a copy) it gets disabled anyway
					// still we want to know if this might be an expansion of something before so we keep on searching...
					if ( 	Tools.isSubsetOf(a_par,new_par)
						  & (sim < PARAGRAPH_SIMILARITY_THRESHOLD) ) {
						sim = 2.0;
					}
					
					// this is in order to avoid duplicates within expansions
					if ( sim >  PARAGRAPH_SIMILARITY_THRESHOLD) {
						have_seen_duplicate = true;
					}

					/* 
					 * Check if this paragraph is an expansion of a former one.
					 * Avoid comparing paragraphs that contain too few words like "He said no!".
					 * Also dont look at very similar paragraphs. They would most likely not contain any new information.
					 */
					if (		(!have_seen_duplicate)
							&	(new_par.tfidf.size() > 5) 
							& 	(a_par.tfidf.size() > 5) 
							&	(Tools.isSubsetOf(new_par,a_par))) {  
						System.out.println(" EEE EEE EEE EEE EEE EEE EEE EEE EEE EEE EEE   expanded paragraph found (NEW / OLD) "+str_sim);
						System.out.println(new_par.getText());
						System.out.println("-------------------");
						System.out.println(a_par.getText());
						System.out.println("");
// @DISABLE for now						a_par.expansions.put(new_par.parent.id, new_par);
					}

					if ( (sim > topsim)  ) {  //&& a_par.isEnabled() @TODO does checking for enabled paragraphs make a lot of difference??
						topsim = sim;
						toppar = a_par;
					}
				}
			}
		}
		
		if (topsim >= 2.0) {
			System.out.println("SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS SSS  subsumed paragraph found (NEW / OLD)");
			System.out.println(new_par.getText());
			System.out.println("-------------------");
			System.out.println(toppar.getText());
			System.out.println("");

			// The information is already covered. Don't bother anymore.
			new_par.disable();

			optimizer.addVar(new_par);  

			return;
		}
		
		String str_sim = String.format("similarity is %.3f",topsim);
		if (topsim > PARAGRAPH_SIMILARITY_THRESHOLD) {
			System.out.println("######################################################################### similar paragraph found (NEW / OLD) "+str_sim);
			System.out.println(new_par.getText());
			System.out.println("-------------------");
			System.out.println(toppar.getText());
			System.out.println("");
			new_par.disable();

			System.out.println("par_id="+toppar.distinct_par_id +" sim="+ topsim);
			
			// since they are similar they are in the same group for optimization (younger paragraphs have their id set by Optimizer)
			new_par.distinct_par_id = toppar.distinct_par_id;
			
			optimizer.addVar(new_par);

			return;
		}

		optimizer.addVar(new_par);
	}


	
	
	public ArrayList<Article> getSimpleArticleList() {
		// copy all articles into a (easy to handle) array
		ArrayList<Article> simple_list = new ArrayList<Article>();
		for (Date d : storyVector.keySet()) {
			for (Article a : storyVector.get(d).articles) {
				simple_list.add(a);
			}
		}
		return simple_list;
	}
	
	


	public void optimize() {
		ArrayList<Article> simple_list = getSimpleArticleList();
		
		// do the magic ...
		optimizer.optimize_article_completenes(simple_list);
		
		// go over all paragraphs and switch ON/OFF according to variables
		for (Article a : simple_list) {
			for (Paragraph p : a.paragraphs) {
				try {
					if (p.opt_var.get(GRB.DoubleAttr.X) == 0.0)
						p.disable();
					else 
						p.enable();
				} catch (GRBException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	


	
	/**
	 * Remove the article a from the collection
	 * @param a article to remove
	 */
	public void removeArticle(Article a) {
		for (Date d : storyVector.keySet()) {
			if (storyVector.get(d).articles.contains(a)) {
				storyVector.get(d).articles.remove(a);
				return;
			}
		}
	}
	
	
	
	
	public void generateTFIDFOnAllArticles() {
		Statistics stats = new Statistics(false);
		BufferedReader br;
		ArrayList<Article> simple_list = getSimpleArticleList();

		// set up TFIDF for all articles in the collection
		for (Article a : simple_list) {
			br = new BufferedReader(new StringReader(a.text));
			stats.readCorpus(br, false); // create per-article stats (count multiple occurrences) 
			for (Integer i : stats.id_to_word.keySet()) {
				a.word_frequencies.put(stats.id_to_word.get(i), stats.id_to_count.get(i));
			}
			a.tfidf = Tools.TFIDFfromWordFrequency(corpus_idf,a.word_frequencies);
			stats.resetStatistics();
		}
	}
	
	
	
	
	/**
	 * Uses the article-based vectors of TFIDF to determine which articles
	 * will be dropped due to high similarity to many other articles.
	 * Using a greedy approach: Take max-distance pair. Make start set from
	 * pair. Add node with highest min-distance to set. Repeat until num 
	 * nodes in set.
	 *
	 * @param num Number of articles that should be kept
	 */
	public void removeDuplicateArticlesBySimilarity(int num) {
		ArrayList<Article> simple_list = getSimpleArticleList();
		ArrayList<Article> result_list = new ArrayList<Article>();
		
		// mapping of similarities between articles
		HashMap<String, HashMap<String,Double> > distances = new HashMap<String, HashMap<String,Double> >();
		
		generateTFIDFOnAllArticles();
		
		// get starting pair while setting up similarities
		Double distmax = 0.0; 
		Article max1=null,max2=null;
		for (Article a1 : simple_list) {
			HashMap<String,Double> temp_map = new HashMap<String,Double>();
			for (Article a2 : simple_list) {
				Double dist = 1.0 - Tools.similarityFromTFIDFVectors(a1, a2); // distance is (1 - cosine_similarity)
				temp_map.put(a2.id, dist);
				if (dist>distmax) { distmax=dist; max1=a1; max2=a2; }
			}
			distances.put(a1.id, temp_map);			
		}
		
		// the 2 that make the starting set are "out"
		result_list.add(max1);
		result_list.add(max2);
		simple_list.remove(max1);
		simple_list.remove(max2);

		// now generate the final set of num articles by expanding on the starting pair
		Double mindist = 1.0;
		Double maxdist = 0.0;
		Article max_dist_rtcl=null;
		Article min_dist_rtcl=null;
		while (result_list.size() < num) {
			// get next node
			maxdist = 0.0;
			for (Article a : simple_list) {
				mindist = 1.0;
				for (Article res_a : result_list) {
					if (distances.get(a.id).get(res_a.id) < mindist) {
						mindist = distances.get(a.id).get(res_a.id);
						min_dist_rtcl = a;
					}
				}
				if (mindist > maxdist) {
					maxdist = mindist;
					max_dist_rtcl = min_dist_rtcl;
				}
			}
			// can drop added node from list (less compares in next round)
			simple_list.remove(max_dist_rtcl);
			result_list.add(max_dist_rtcl);
			max_dist_rtcl.enable();
		}
		
		System.out.println("result list size is "+result_list.size());
		
		// reset the simple list (all articles in story vector)
		simple_list = getSimpleArticleList();
		
		
		// setting up statistics for summary-coverage-ratio SCR
		for (Article a : simple_list)
			for (Paragraph p : a.paragraphs) {
				p.enable();  // first disable everything
			}
		for (Article a : result_list)
			for (Paragraph p : a.paragraphs) {
				p.enable();  // enable the selected articles
			}
		// now all our selected articles are 100% enabled the rest is not
		// this allows us to simply call the same function for SCR as with optimization

		
		// now get rid of all articles (remove from storyVector) that are not in our result set
		for (Article a : simple_list) {
			if (!result_list.contains(a)) {
				removeArticle(a);
			}else {  // if its a selected one then enable it completely
				for (Paragraph p : a.paragraphs) {
					p.enable();
				}
			}
		}
		
	}
	
	
	
	
	
	
	
	public void evaluateByGroupDistance() {
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
		ArrayList<Article> selected_articles = new ArrayList<Article>();

		generateTFIDFOnAllArticles();
		
		// sort out selected articles
		for (Date d : storyVector.keySet()) {
			for (Article a : storyVector.get(d).articles) {
				System.out.format("article enabled pctn %.3f",a.enabledPercentage());
				if (a.enabledPercentage() > MIN_PERCENT_TO_ENABLE_ARTICLE) {
					selected_articles.add(a);	
				}
			}
		}
		
		System.out.println("\n");
		int count = selected_articles.size();
		double min=1.0,max=0,sum=0,avg=0;
		Iterator<Article> a_iter = selected_articles.iterator();
		while (a_iter.hasNext()) {
			Article a1 = a_iter.next();
			for (Article a2 : selected_articles) {
				Double dist = 1.0 - Tools.similarityFromTFIDFVectors(a1, a2); // distance is (1 - cosine_similarity)
				System.out.format("dist=%.3f \n",dist);
				sum += dist;
				if (dist<min) min = dist;
				if (dist>max) max = dist;
			}
			a_iter.remove();
		}
		
		// distance count is artcles count gaussian sum (node-count -> edge-count)
		avg = sum/(count*(count+1)/2);
		
		System.out.format("avg=%.3f min=%.3f max=%.3f count=%d\n",avg,min,max,count);
	}
	
	
	
	
	
	
	
	/**
	 * Dump all the collected articles to 2 HTML files. "index.html" is a simple
	 * graphical overview. "text.html" is the text of all articles colored according
	 * to enabled/disabled duplicates (duplicate paragraphs). "summary.html" is a 
	 * brief overview of the most complete articles.
	 */
	public void printHtml() {
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
		String bgcol = "";
		boolean printAsOverview = false;
		int rtcl_en,rtcl_dis;

		try {

            FileWriter fw_simstats = new FileWriter("similarity_stats.csv",false);
			for (int i=0; i<100; i++) {
				fw_simstats.write(i/100.0+"  "+simstats[i]+"\n");
			}
			fw_simstats.close();
			
			/** 
			 * Now we do SCR and SDR
			 */
			getCoverageRatioOfSummary();
			getDuplicateRatioOfSummary();
			
			FileWriter fw_stats = new FileWriter("paragraph_stats.csv",false);
			fw_stats.write("which articles have been chosen as summary articles ...\n\n");
			
			FileWriter fw_graphical = new FileWriter("index.html",false);
    		fw_graphical.write("<HTML><BODY><BR/>\n");
    		fw_graphical.write("<a href=\"summary.html\">See the summary articles</a><br/><br/>");

    		FileWriter fw_text = new FileWriter("text.html",false);
    		fw_text.write("<HTML><HEAD>\n");
    		fw_text.write("<script type='text/javascript' src='jquery-1.10.2.js'></script>\n");
    		fw_text.write("<script src=\"jquery-ui-1.10.3.js\"></script>\n");
    		fw_text.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"jquery-ui-1.10.3.css\">\n");
    		fw_text.write("<script type='text/javascript' src='bubble.js'></script>\n");
    		fw_text.write("</HEAD>\n<BODY>\n<BR/>\n");

    		FileWriter fw_sum = new FileWriter("summary.html",false);
    		fw_sum.write("<HTML><HEAD>\n");
    		fw_sum.write("<script type='text/javascript' src='jquery-1.10.2.js'></script>\n");
    		fw_sum.write("<script src=\"jquery-ui-1.10.3.js\"></script>\n");
    		fw_sum.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"jquery-ui-1.10.3.css\">\n");
    		fw_sum.write("<script type='text/javascript' src='bubble.js'></script>\n");
    		fw_sum.write("</HEAD>\n<BODY>\n<BR/>\n");

    		for (Date d : storyVector.keySet()) {
    			fw_graphical.write("<BR/>"+dateFormat.format(d)+"<BR/>");
    			fw_text.write("<BR/>"+dateFormat.format(d)+"<BR/>");
    			for (Article a : storyVector.get(d).articles) {
    				rtcl_en=0; rtcl_dis=0;
    				// if > 90% enabled AND not just one line of text -> show in overview
    				printAsOverview = (a.enabledPercentage() > MIN_PERCENT_TO_ENABLE_ARTICLE); 
    				/*O*/fw_text.write("<B><U>"+a.headline+"</U></B><BR/>\n("+a.id+")<BR/>\n");
    				/*O*/if (printAsOverview) fw_sum.write("<B><U>"+a.headline+"</U></B><BR/>\n("+a.id+")<BR/>\n");  //overview
    				for (Paragraph a_par : a.paragraphs) {
    					int s = 6 + a_par.tfidf.size();
    					String refname = a_par.refname();
    					bgcol = ((a_par.pnum&1) == 0) ? "F2FFFF" : "FFF2FF";
						String exp = "";
    					if (!a_par.expansions.keySet().isEmpty()) {
   						for (String ep_id : a_par.expansions.keySet()) { exp += ",\""+a_par.expansions.get(ep_id).refname()+"\""; }
    						exp = exp.replaceFirst("," , "[");
    						exp += "]";
    					}
    					fw_text.write("<a name=\""+refname+"\"><span class=\"bubble\" id=\""+refname+"\" exp='"+exp+"'>\n");
    					/*O*/if (printAsOverview) fw_sum.write("<a name=\""+refname+"\"><span id=\""+refname+"\">\n");
//						/*O*/if (printAsOverview) fw_sum.write("<font color=black style=\"background-color:#"+bgcol+"\">");
    					if (a_par.isEnabled()) {
    						fw_text.write("<font color=black style=\"background-color:#"+bgcol+"\">");
    						if (printAsOverview) fw_sum.write("<font color=black style=\"background-color:#"+bgcol+"\">");
    						fw_graphical.write("<a href=\"text.html#"+refname+"\"><IMG width="+s+" height="+s+" SRC=\"green1.png\"/></a>\n");
    					} else {
    						fw_text.write("<font color=silver style=\"background-color:#"+bgcol+"\">");
    						if (printAsOverview) fw_sum.write("<font color=silver style=\"background-color:#"+bgcol+"\">");
    						fw_graphical.write("<a href=\"text.html#"+refname+"\"><IMG width="+s+" height="+s+" SRC=\"clear.png\"/></a>\n");
    					}
    					fw_text.write(a_par.pnum+") ");
    					/*O*/if (printAsOverview) fw_sum.write(a_par.pnum+") ");
    					if (exp.length() > 5) { // when expansions exist
    						fw_text.write("<img src=\"e.png\"/>");
    					}
    					fw_text.write(a_par.getText()+"<BR/></font>\n");
    					fw_text.write("</span></a>");
    					/*O*/if (printAsOverview) fw_sum.write(a_par.getText()+"<BR/></font>\n");
    					/*O*/if (printAsOverview) fw_sum.write("</span></a>");
    					
    					// stats
    					if (a_par.isEnabled())
    						rtcl_en += a_par.tfidf.size();
    					else 
    						rtcl_dis += a_par.tfidf.size();
    				}
    				fw_text.write("<BR/><BR/>\n");
    				/*O*/if (printAsOverview) fw_sum.write("<BR/><BR/>\n");
    				fw_graphical.write("<BR/>\n");

    				System.out.println("paragraph stats write ........");
    				fw_stats.write(rtcl_en+"\t"+rtcl_dis+"\t"+(printAsOverview?(rtcl_dis+rtcl_en):0)+"\n");
    			}
    		}

    		fw_text.write("<div id=\"dialog\"></div></BODY></HTML>\n");
    		fw_text.close();

    		fw_sum.write("<div id=\"dialog\"></div></BODY></HTML>\n");
    		fw_sum.close();

    		fw_graphical.write("</BODY></HTML>\n");
    		fw_graphical.close();
    		
    		fw_stats.close();
    		
    	} catch (IOException e) {
    		throw new RuntimeException(e);
    	}		
	}
	
	
	/**
	 * Based on the parsed paragraphs and their grouping into distinct sets of
	 * "similar paragraphs" or "copies", this method counts how many of the overall
	 * distinct paragraphs are being covered within a given summary.
	 * @return
	 */
	public void getCoverageRatioOfSummary() {
		ArrayList<Article> simple_list = getSimpleArticleList();
		Integer distinct_paragraph_count = 0;  // IDs start with 0 and are numbered consecutively
		Integer dist_pars_in_summary = 0;
		TreeSet<Integer> par_ids = new TreeSet<Integer>();
		
		for (Article a : simple_list) {
			for (Paragraph p : a.paragraphs) {
				if (p.distinct_par_id > distinct_paragraph_count) 
					distinct_paragraph_count = p.distinct_par_id;
				if (a.isEnabled()) {
//					System.out.println("found another summary dist_par_id "+p.distinct_par_id+" in "+p.parent.id);
					par_ids.add(p.distinct_par_id);
				}
			}
		}
		
		dist_pars_in_summary = par_ids.size();
		System.out.println("SCR: distinct in summary = "+dist_pars_in_summary);
		System.out.println("SCR: distinct overall = "+distinct_paragraph_count);
		Double ratio = dist_pars_in_summary.doubleValue() / distinct_paragraph_count.doubleValue();
		System.out.println("SCR: ratio SCR="+ratio+"\n");			
		return;
	}


	/**
	 * Get a ratio of 
	 * (distinct paragraphs in the summary) / (all paragaphs in the summary)
	 * such that it should be 1.0 if the summary is made up of only distinct
	 * and unique paragraphs and if its all copies (all the same paragraphs)
	 * it should be 0.0
	 */
	public void getDuplicateRatioOfSummary() {
		ArrayList<Article> simple_list = getSimpleArticleList();
		Integer summary_paragraph_count = 0;
		TreeSet<Integer> par_ids = new TreeSet<Integer>();
		
		for (Article a : simple_list) {
			for (Paragraph p : a.paragraphs) {
				if (a.isEnabled()) {
					par_ids.add(p.distinct_par_id);
					summary_paragraph_count++;
				}
			}
		}
		
		System.out.println("SDR: any in summary = "+summary_paragraph_count);
		System.out.println("SDR: distinct in summary = "+par_ids.size());
		Double temp = new Double(par_ids.size());
		Double ratio = temp / summary_paragraph_count.doubleValue();
		System.out.println("SDR: ratio SDR="+ratio+"\n");			
		return;
	}

	
	public LinkedList<String> getSummaryIDs()  {
		LinkedList<String> lst = new LinkedList<String>();
		ArrayList<Article> simple_list = getSimpleArticleList();
		
		for (Article a : simple_list) {
			if (a.enabledPercentage() > MIN_PERCENT_TO_ENABLE_ARTICLE) {
				lst.add(a.id);
			}
		}
		System.out.println("Summary article count = "+lst.size());
		return lst;
	}
	public void setSummaryIDs(LinkedList<String> summaryIDs) {
		ArrayList<Article> simple_list = getSimpleArticleList();
		for (Article a : simple_list) {
			a.disable();
		}
		for (String s : summaryIDs) {
			System.out.println("enabling ID "+s);
			for (Article a : simple_list) {
				if (a.id.equals(s)) {
					a.enable();
					System.out.println("found article for ID "+s);
				}
			}
		}
	}
	
	
	/**
	 * Only used in evaluation.
	 * Sets the "enabled" flag on all articles with the
	 * required precentage of paragraphs (this is used to
	 * carry over the state of summary articles into the evaluation
	 * phase which may use a different threshold for paragaph
	 * similarity and re-run the analysis (hence change the
	 * percentage of paragraphs in the articles).
	 */
	public void setSummaryArticleState() {
		ArrayList<Article> simple_list = getSimpleArticleList();
		int summary_article_count = 0;
		
		for (Article a : simple_list) {
			if (a.enabledPercentage() > MIN_PERCENT_TO_ENABLE_ARTICLE) {
				a.enable();
				summary_article_count++;
			} else {
				a.disable();
			}
		}
		System.out.println("Summary article count = "+summary_article_count);
	}
	
	
	/**
	 * Only used in evaluation.
	 */
	public void resetOptimizationState() {
		ArrayList<Article> simple_list = getSimpleArticleList();
		
		optimizer = new Optimizer();
		for (Article a : simple_list) {
			for (Paragraph p : a.paragraphs) {
				p.distinct_par_id = 0;
				p.opt_var = null;
			}
		}
	}
	
	
	/**
	 * Only used in evaluation.
	 * Change the paragaph similarity threshold
	 */
	public void setPARAGRAPH_SIMILARITY_THRESHOLD (Double d) {
		PARAGRAPH_SIMILARITY_THRESHOLD = d;
	}
	public Double getPARAGRAPH_SIMILARITY_THRESHOLD () {
		return PARAGRAPH_SIMILARITY_THRESHOLD;
	}


}







